searchData={"content_type":"text/markdown","items":[{"doc":"Use this module in any module that uses `Ecto.Repo`\nto enrich it with `cursor_stream/2` function.\n\nExample:\n\n    defmodule MyRepo do\n      use Ecto.Repo\n      use Cursornator\n    end\n\n    MyUser\n    |> MyRepo.cursor_stream(max_rows: 100)\n    |> Stream.each(...)\n    |> Stream.run()","ref":"Cursornator.html","title":"Cursornator","type":"behaviour"},{"doc":"","ref":"Cursornator.html#c:cursor_query/2","title":"Cursornator.cursor_query/2","type":"callback"},{"doc":"Return a lazy enumerable that emits all entries from the data store\nmatching the given query.\n\nIn contrast to `Ecto.Repo.stream/2`,\nthis will not use database mechanisms (e.g. database transactions) to stream the rows.\n\nIt does so by sorting all the rows by `:cursor_field` and iterating over them in chunks\nof size `:max_rows`.","ref":"Cursornator.html#c:cursor_stream/2","title":"Cursornator.cursor_stream/2","type":"callback"},{"doc":"* `:cursor_field` - the field or list of fields by which all rows should be iterated.\n\n  **This field must have unique values. (Otherwise, some rows may get skipped.)**\n\n  For performance reasons, we recommend that you have an index on that field. Defaults to `:id`.\n\n* `:after_cursor` - the value of the `:cursor_field` that results start. When `:cursor_field` is a list\n  then`:after_cursor` must be a map where keys are cursor fields (not all fields are required).\n\n  Useful when you want to continue streaming from a certain point.\n  Any rows with value equal or smaller than this value will not be included.\n\n  Defaults to `nil`. (All rows will be included.)\n\n* `:max_rows` - The number of rows to load from the database as we stream.\n\n  Defaults to 500.\n\n* `:order` - Order of results, `:asc` or `:desc`\n\n  Defaults to `:asc`. If list of cursor fields is given with specific order, then this option is ignored.\n\n* `:parallel` - when `true` fetches next batch of records in parallel to processing the stream.\n\n  Defaults to `false` as this spawns `Task`s and could cause issues e.g. with Ecto Sandbox in tests.\n\n* `:prefix, :timeout, :log, :telemetry_event, :telemetry_options` - options passed directly to `Ecto.Repo.all/2`","ref":"Cursornator.html#c:cursor_stream/2-options","title":"Options - Cursornator.cursor_stream/2","type":"callback"},{"doc":"MyUser\n    |> MyRepo.cursor_stream(max_rows: 1000)\n    |> Stream.each(...)\n    |> Stream.run()\n\n    # change order, run in parallel\n    MyUser\n    |> MyRepo.cursor_stream(order: :desc)\n    |> Stream.each(...)\n    |> Stream.run()\n\n    # change cursor field and set starting cursor\n    MyUser\n    |> MyRepo.cursor_stream(cursor_field: :email, after_cursor: \"foo@bar.com\")\n    |> Stream.each(...)\n    |> Stream.run()\n\n    # with multiple fields\n    MyUser\n    |> MyRepo.cursor_stream(cursor_field: [:email, :date_of_birth], after_cursor: %{email: \"foo@bar.com\"})\n    |> Stream.each(...)\n    |> Stream.run()\n\n    # Multi field cursor with custom order\n    MyUser\n    |> MyRepo.cursor_stream(cursor_field: [email: :desc, id: :asc], order: :desc, after_cursor: %{email: \"foo@bar.com\", id: 1})\n    |> Stream.each(...)\n    |> Stream.run()\n\n    # select custom fields, remember to add cursor_field to select\n    MyUser\n    |> select([u], map(u, [:my_id, ...])\n    |> select_merge([u], ...)\n    |> MyRepo.cursor_stream(cursor_field: :my_id)\n    |> Stream.each(...)\n    |> Stream.run()\n\n    # pass custom options to Ecto.Repo.all/2\n    MyUser\n    |> MyRepo.cursor_stream(timeout: 60_000, prefix: \"public\")\n    |> Stream.each(...)\n    |> Stream.run()","ref":"Cursornator.html#c:cursor_stream/2-examples","title":"Examples - Cursornator.cursor_stream/2","type":"callback"},{"doc":"","ref":"Cursornator.html#t:cursor_opts/0","title":"Cursornator.cursor_opts/0","type":"type"},{"doc":"# Cursornator\n\n\n> **Notes**: This repo is my customized version of [Ecto Cursor based stream](https://github.com/allegro/ecto-cursor-based-stream) for using in my projects.\n\nCursor-based streaming of Ecto records, that does not require database transaction.\n\nAdvantages in comparison to the standard `Ecto.Repo.stream/2`:\n\n- streaming can be stopped and continued at any point (by passing option `after_cursor: ...`),\n- works with tables that have milions of records.\n\nOnly limitation is that you have to supply a _cursor column or columns_ (by passing option `cursor_field: ...`, defaults to `:id`). Such a column(s):\n\n- must have unique values,\n- should have a database index. (So that sorting by it, and returning a number of rows larger than `x` is a performant operation.)","ref":"readme.html","title":"Cursornator","type":"extras"},{"doc":"1. Add `cursornator` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:cursornator, \"~> 1.3.0\"}\n  ]\nend\n```\n\n2. Add `use Cursornator` to the module that uses `Ecto.Repo`:\n\n```elixir\ndefmodule MyRepo do\n  use Ecto.Repo\n  use Cursornator\nend\n```\n\n3. Stream the rows using `cursor_stream/2`:\n\n```elixir\nPost\n|> MyRepo.cursor_stream(max_rows: 100)\n|> Stream.each(...)\n|> Stream.run()\n```\n\n4. Query if you don't want to stream\n\n```elixir\n{posts, next_cursor} = MyRepo.cursor_query(Post, cursor_field: [published_at: :desc, id: :desc])\n```","ref":"readme.html#usage","title":"Usage - Cursornator","type":"extras"}],"producer":{"name":"ex_doc","version":[48,46,51,50,46,50]}}